

--- Base class for Mission Tasks.
-- This class defines the base class for Tasks.
-- @field Task.SIGNAL base type to express signals to be fired upon task stages.
-- @field Task.ClassName Name of the Class.
TASK = {

  -- Defines the different signal types with a Task.
  SIGNAL = {
    COLOR = { 
      RED = { ID = 1, COLOR = trigger.smokeColor.Red, TEXT = "A red" },
      GREEN = { ID = 2, COLOR = trigger.smokeColor.Green, TEXT = "A green" }, 
      BLUE = { ID = 3, COLOR = trigger.smokeColor.Blue, TEXT = "A blue" },
      WHITE = { ID = 4, COLOR = trigger.smokeColor.White, TEXT = "A white" }, 
      ORANGE = { ID = 5, COLOR = trigger.smokeColor.Orange, TEXT = "An orange" } 
    },
    TYPE = {
      SMOKE = { ID = 1, TEXT = "smoke" },
      FLARE = { ID = 2, TEXT = "flare" }
    }
  },
  ClassName = "Task",
  Name = '',
  Stages = {},
  Stage = {},
  ActiveStage = 0,
  TaskDone = false,
  TaskFailed = false
}

function TASK:New()
  env.info( "Task:New()" )

  local Child = BASE:Inherit( self, BASE:New() )

  -- assign Task default values during construction
  Child.TaskBriefing = "Task: No Task."
  Child.Time = timer.getTime()
  Child.ExecuteStage = _TransportExecuteStage.NONE

  return Child
end

function TASK:SetStage( StageSequenceIncrement )
--  env.info( "Task:SetStage(" .. self.ClassName .. ", " .. StageSequenceIncrement .. ")" )

	if StageSequenceIncrement ~= 0 then
		self.ActiveStage = self.ActiveStage + StageSequenceIncrement
		if 1 <= self.ActiveStage and self.ActiveStage <= #self.Stages then
			self.Stage = self.Stages[self.ActiveStage]
			env.info( "Task:SetStage(): self.Stage.Name = " .. self.Stage.Name )
			self.Frequency = self.Stage.Frequency
		else
			error( "TASK:SetStage() self.ActiveStage is smaller or larger than self.Stages array. self.ActiveStage = " .. self.ActiveStage )
		end
	end
	self.Time = timer.getTime()
end

function TASK:Init()
	self.ActiveStage = 0
	self:SetStage(1)
	self.TaskDone = false
	self.TaskFailed = false
end

function TASK:Done()
  self.TaskDone = true
end

function TASK:IsDone()
  return self.TaskDone
end

function TASK:Fail()
  env.info( "Task:Fail()" )
  self.TaskFailed = true
end

function TASK:HasFailed( ClientDead )
  return self.TaskFailed
end

function TASK:Reset( Mission, Client )
  self.ExecuteStage = _TransportExecuteStage.NONE
end


function TASK.MenuAction( Parameter )
  env.info( "TASK:MenuAction()" )
  
  env.info( Parameter.ReferenceTask.ClassName )
  env.info( Parameter.CargoName )
  Parameter.ReferenceTask.ExecuteStage = _TransportExecuteStage.EXECUTING
  Parameter.ReferenceTask.CargoName = Parameter.CargoName
  
end

function TASK:StageExecute()
--  env.info( "TASK:StageExecute(): self.Stage.Name = " .. self.Stage.Name .. ", self.Frequency = " .. self.Frequency )

  local Execute = false

  if      self.Frequency == STAGE.FREQUENCY.REPEAT then
    Execute = true
  elseif  self.Frequency == STAGE.FREQUENCY.NONE then
    Execute = false
  elseif  self.Frequency >= 0 then
    Execute = true
    self.Frequency = self.Frequency - 1
  end
  
  return Execute

end

--- Method
-- Work function to set signal events within a TASK.
-- @see AddSmokeRed
-- @see AddSmokeGreen
function TASK:AddSignal( SignalVehicleGroupName, SignalType, SignalColor, SignalHeight )
	env.info( "TASK:AddSignal(): " .. SignalVehicleGroupName )
  
	local Valid = true
	local SignalUnit
	
	Valid = _TransportValidateGroup( SignalVehicleGroupName, "SignalVehicleGroupName", Valid )

	if Valid then
		self.LandingZoneSignalVehicle = SignalVehicleGroupName
		self.LandingZoneSignalType = SignalType
		self.LandingZoneSignalColor = SignalColor
		if SignalHeight ~= nil then
			self.LandingZoneSignalHeight = SignalHeight
		else
			self.LandingZoneSignalHeight = 0 
		end
	  
		if self.TaskBriefing then 
			local SignalGroup = Group.getByName( SignalVehicleGroupName )
			if SignalGroup == nil then
				SignalUnit = StaticObject.getByName( SignalVehicleGroupName )
			else
				SignalUnit = SignalGroup:getUnits()[1]
			end
			if SignalUnit:getLife() <= 1.0 then
				self.TaskBriefing = self.TaskBriefing .. " No signal will be provided upon arrival as the signalling unit " .. SignalVehicleGroupName .. " cannot be reached."
			else
				self.TaskBriefing = self.TaskBriefing .. " " .. SignalColor.TEXT .. " " .. SignalType.TEXT .. " will be fired when entering the landing zone by the " .. SignalVehicleGroupName
			end
		end
	end
	
	return Valid
end

--- Method
-- When the CLIENT is approaching the landing zone, a RED SMOKE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddSmokeRed( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.SMOKE, TASK.SIGNAL.COLOR.RED, SignalHeight )
end

--- Method
-- When the CLIENT is approaching the landing zone, a GREEN SMOKE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddSmokeGreen( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.SMOKE, TASK.SIGNAL.COLOR.GREEN, SignalHeight )
end
        
--- Method
-- When the CLIENT is approaching the landing zone, a BLUE SMOKE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddSmokeBlue( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.SMOKE, TASK.SIGNAL.COLOR.BLUE, SignalHeight )
end

--- Method
-- When the CLIENT is approaching the landing zone, a WHITE SMOKE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddSmokeWhite( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.SMOKE, TASK.SIGNAL.COLOR.WHITE, SignalHeight )
end

--- Method
-- When the CLIENT is approaching the landing zone, an ORANGE SMOKE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddSmokeOrange( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.SMOKE, TASK.SIGNAL.COLOR.ORANGE, SignalHeight )
end

--- Method
-- When the CLIENT is approaching the landing zone, a RED FLARE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddFlareRed( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.FLARE, TASK.SIGNAL.COLOR.RED, SignalHeight )
end

--- Method
-- When the CLIENT is approaching the landing zone, a GREEN FLARE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddFlareGreen( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.FLARE, TASK.SIGNAL.COLOR.GREEN, SignalHeight )
end
        
--- Method
-- When the CLIENT is approaching the landing zone, a BLUE FLARE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddFlareBlue( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.FLARE, TASK.SIGNAL.COLOR.BLUE, SignalHeight )
end

--- Method
-- When the CLIENT is approaching the landing zone, a WHITE FLARE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddFlareWhite( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.FLARE, TASK.SIGNAL.COLOR.WHITE, SignalHeight )
end

--- Method
-- When the CLIENT is approaching the landing zone, an ORANGE FLARE will be fired by an optional SignalVehicleGroupName.
-- @param self TASK object.
-- @param SignalVehicleGroupName Name of the Group that will fire the signal. If this parameter is NIL, the signal will be fired from the center of the landing zone.
function TASK:AddFlareOrange( SignalVehicleGroupName, SignalHeight )
  self:AddSignal( SignalVehicleGroupName, TASK.SIGNAL.TYPE.FLARE, TASK.SIGNAL.COLOR.ORANGE, SignalHeight )
end

--- Class
-- Modeling a sequence of STAGEs to unload cargo within a specified Landing Zone. 
DEPLOYTASK = {
  ClassName = "DEPLOYTASK",
  TEXT = { "Deploy", "deployed", "unloaded" }
}

--- Constructor
-- Creates a new DEPLOYTASK object, which models the sequence of STAGEs to unload a cargo.
-- @param self DEPLOYTASK object
-- @param LandingZoneName Name of the Zone where Cargo is to be unloaded.
-- @param CargoName Cargo name. This name must be unique as it is a unique search key to identify the different cargo elements within the mission.
-- @param CargoType Type of the Cargo.
-- @author svenvandevelde
function DEPLOYTASK:New( LandingZoneName, CargoType )
  env.info( "DeployTask:New()" )

  -- Child holds the inherited instance of the DEPLOYTASK Class to the BASE class.
  local Child = BASE:Inherit( self, TASK:New() )
  
  local Valid = true
  
  Valid = _TransportValidateZone( LandingZoneName, "LandingZoneName", Valid )
  Valid = _TransportValidateEnum( CargoType, "CargoType", _TransportType, Valid )
    
  if Valid then
    Child.Name = 'DeployTask'
    Child.TaskBriefing = "Task: Fly to " .. LandingZoneName .. " and deploy " .. _TransportTypeText[CargoType] .. ". Your co-pilot will provide you with the directions (required flight angle in degrees) and the distance (in km) to the deployment zone."
    Child.LandingZoneName = LandingZoneName
    Child.CargoType = CargoType
    Child.Stages = { STAGEBRIEF:New(), STAGESTART:New(), STAGEROUTE:New(), STAGELANDING:New(), STAGELANDED:New(), STAGEUNLOAD:New(), STAGEDONE:New() }
    Child:SetStage(1)
  end
  
  return Child
end

--- Method
-- When the cargo is unloaded, it will move to the target zone name.
-- @param self DEPLOYTASK object
-- @param TargetZoneName Name of the Zone to where the Cargo should move after unloading.
function DEPLOYTASK:SetCargoTargetZoneName( TargetZoneName )
  env.info( "DeployTask:SetCargoTargetZoneName()" )
  
  local Valid = true
  
  Valid = _TransportValidateString( TargetZoneName, "TargetZoneName", Valid )
  
  if Valid then
    self.TargetZoneName = TargetZoneName
  end
  
  return Valid
  
end

function DEPLOYTASK:AddCargoMenus( Client, Cargos, TransportRadius )
	env.info( 'DEPLOYTASK:AddCargoMenus()' )

	for CargoID, CargoData in pairs( Client._Cargos ) do

		env.info( 'DEPLOYTASK:AddCargoMenus() CargoData.CargoName = ' .. CargoData.CargoName )
		if Client._Menus[CargoData.CargoType] == nil then
			Client._Menus[CargoData.CargoType] = {}
		end
		
		if Client._Menus[CargoData.CargoType].DeployMenu == nil then
			Client._Menus[CargoData.CargoType].DeployMenu = missionCommands.addSubMenuForGroup(
				Client:ClientGroup():getID(), 
				self.TEXT[1], 
				nil
			)
			env.info( 'DEPLOYTASK:AddCargoMenus(): Added DeployMenu ' .. self.TEXT[1] )
		end
		
		if Client._Menus[CargoData.CargoType].DeploySubMenus == nil then
			Client._Menus[CargoData.CargoType].DeploySubMenus = {}
		end
		
		if Client._Menus[CargoData.CargoType].DeployMenu == nil then
			env.info('deploymenu is nil')
		end

		Client._Menus[CargoData.CargoType].DeploySubMenus[ #Client._Menus[CargoData.CargoType].DeploySubMenus + 1 ].MenuPath = missionCommands.addCommandForGroup(
			Client:ClientGroup():getID(), 
			_TransportTypeText[CargoData.CargoType] .. ":" .. CargoData.CargoName .. " ( " .. CargoData.CargoWeight .. "kg )",
			Client._Menus[CargoData.CargoType].DeployMenu, 
			self.MenuAction,
			{ ReferenceTask = self, CargoName = CargoData.CargoName }
		)
			env.info( 'DEPLOYTASK:AddCargoMenus(): Added DeploySubMenu ' .. _TransportTypeText[CargoData.CargoType] .. ":" .. CargoData.CargoName .. " ( " .. CargoData.CargoWeight .. "kg )" )
	end

end

function DEPLOYTASK:RemoveCargoMenus( Client )
	env.info( "DEPLOYTASK:RemoveCargoMenus()" )

	for MenuID, MenuData in pairs( Client._Menus ) do
		if MenuData.DeploySubMenus ~= nil then
			for SubMenuID, SubMenuData in pairs( MenuData.DeploySubMenus ) do
				missionCommands.removeItemForGroup( Client:ClientGroup():getID(), SubMenuData )
				env.info( "DEPLOYTASK:RemoveCargoMenus(): Removed DeploySubMenu " )
				SubMenuData = nil
			end
		end
		if MenuData.DeployMenu then
			missionCommands.removeItemForGroup( Client:ClientGroup():getID(), MenuData.DeployMenu )
			env.info( "DEPLOYTASK:RemoveCargoMenus(): Removed DeployMenu " )
			MenuData.DeployMenu = nil
		end
	end

end

--- Class
-- Modeling a sequence of STAGEs to load cargo from a specified Landing Zone.
PICKUPTASK = {
  ClassName = "PICKUPTASK",
  TEXT = { "Pick-Up", "picked-up", "loaded" }
}

--- Constructor
-- Creates a new PICKUPTASK.
-- @param self
-- @param LandingZoneName Name of the Zone where Cargo is to be loaded.
-- @param CargoName Cargo name. This name must be unique as it is a unique search key to identify the different cargo elements within the mission.
-- @param CargoGroupName Name of the Group that is represented as the cargo. This Group will be moving towards the Carrier when receiving the command to be picked-up. 
-- @param CargoType Type of the Cargo. The type must be of the following Enumeration:..
-- @param CargoWeight This number indicates the weight in kilograms of the cargo. Note that this parameter cannot reflect yet the total weight within the Carriers.
-- @param OnBoardSide Reflects from which side the cargo Group will be on-boarded on the Carrier.
-- @param ParentClass (Optional) Used to pass the master class within a constructor.
-- @author svenvandevelde
function PICKUPTASK:New( LandingZoneName, CargoType, OnBoardSide )
  env.info( "PickupTask:New()" .. self.ClassName )

  -- Child holds the inherited instance of the PICKUPTASK Class to the BASE class.
  local Child = BASE:Inherit( self, TASK:New() )

  local Valid = true
  
  Valid = _TransportValidateZone( LandingZoneName, "LandingZoneName", Valid )
  Valid = _TransportValidateEnum( CargoType, "CargoType", _TransportType, Valid )
  Valid = _TransportValidateEnum( OnBoardSide, "OnBoardSide", CLIENT.ONBOARDSIDE, Valid )
    
  if  Valid then
    Child.Name = 'PickupTask'
    Child.TaskBriefing = "Task: Fly to " .. LandingZoneName .. " and pickup " .. _TransportTypeText[CargoType] .. ". Your co-pilot will provide you with the directions (required flight angle in degrees) and the distance (in km) to the pickup zone."
    Child.LandingZoneName = LandingZoneName
    Child.CargoType = CargoType
    Child.OnBoardSide = OnBoardSide
    Child.Stages = { STAGEBRIEF:New(), STAGESTART:New(), STAGEROUTE:New(), STAGELANDING:New(), STAGELANDED:New(), STAGELOAD:New(), STAGEDONE:New() }
    Child:SetStage(1)
  end
  
  return Child
end

function PICKUPTASK:AddCargoMenus( Client, Cargos, TransportRadius )
	env.info( 'PICKUPTASK:AddCargoMenus()' )
  
	for CargoID, CargoData in pairs( Cargos ) do

		if CargoData.Status ~= CARGOSTATUS.LOADED and CargoData.Status ~= CARGOSTATUS.LOADING then
		
			if Group.getByName( CargoData.CargoGroupName ) then
			
				if Group.getByName( CargoData.CargoGroupName ):getSize() >= 1 then
			
					if Client._Menus[CargoData.CargoType] == nil then
						Client._Menus[CargoData.CargoType] = {}
					end
					
					if Client._Menus[CargoData.CargoType].PickupMenu == nil then
						Client._Menus[CargoData.CargoType].PickupMenu = missionCommands.addSubMenuForGroup(
							Client:ClientGroup():getID(), 
							self.TEXT[1], 
							nil
						)
						env.info( 'PICKUPTASK:AddCargoMenus(): Added PickupMenu' .. self.TEXT[1] )
					end

					if Client._Menus[CargoData.CargoType].PickupSubMenus == nil then
						Client._Menus[CargoData.CargoType].PickupSubMenus = {}
					end
					
					local MenuAdd = false
					if CargoData.CargoType == _TransportType.INFANTRY then
						CargoGroup = Group.getByName( CargoData.CargoGroupName )
						if _TransportValidateGroupUnitsInRadius( CargoGroup, Client:ClientGroup(), TransportRadius ) then
							MenuAdd = true
						end
					else
						MenuAdd = true
					end
					
					if MenuAdd then
						Client._Menus[CargoData.CargoType].PickupSubMenus[ #Client._Menus[CargoData.CargoType].PickupSubMenus + 1 ] = missionCommands.addCommandForGroup(
							Client:ClientGroup():getID(), 
							_TransportTypeText[CargoData.CargoType] .. ":" .. CargoData.CargoName .. " ( " .. CargoData.CargoWeight .. "kg )",
							Client._Menus[CargoData.CargoType].PickupMenu, 
							self.MenuAction,
							{ ReferenceTask = self, CargoName = CargoData.CargoName }
						)
						env.info( 'PICKUPTASK:AddCargoMenus(): Added PickupSubMenu' .. _TransportTypeText[CargoData.CargoType] .. ":" .. CargoData.CargoName .. " ( " .. CargoData.CargoWeight .. "kg )" )
					end
				end
			end
		end
	end
	
end

function PICKUPTASK:RemoveCargoMenus( Client )
	env.info( "PICKUPTASK:RemoveCargoMenus()" )

	for MenuID, MenuData in pairs( Client._Menus ) do
		for SubMenuID, SubMenuData in pairs( MenuData.PickupSubMenus ) do
			missionCommands.removeItemForGroup( Client:ClientGroup():getID(), SubMenuData )
			env.info( "PICKUPTASK:RemoveCargoMenus(): Removed PickupSubMenu " )
			SubMenuData = nil
		end
		if MenuData.PickupMenu then
			missionCommands.removeItemForGroup( Client:ClientGroup():getID(), MenuData.PickupMenu )
			env.info( "PICKUPTASK:RemoveCargoMenus(): Removed PickupMenu " )
			MenuData.PickupMenu = nil
		end
	end
			
end



function PICKUPTASK:HasFailed( ClientDead )

	local TaskHasFailed = self.TaskFailed
--	env.info( 'PICKUPTASK:HasFailed() start : ' .. string.format ( "%s", TaskHasFailed and "true" or "false" ) )

--	if ClientDead then
--		if type( self.CargoGroupName ) == 'table' then
--			for CargoItem = #self.CargoGroupName, 1, -1 do
--				if Group.getByName(self.CargoGroupName[CargoItem]):getSize() == 0 then
--					table.remove( self.CargoGroupName, CargoItem )
--				end
--			end
--			if #self.CargoGroupName == 0 then
--				env.info('PICKUPTASK:HasFailed() no cargo left. fail task.')
--				TaskHasFailed = true 
--			end
--		else
--			local CargoGroup = Group.getByName( self.CargoGroupName )
--			if CargoGroup and CargoGroup:getSize() > 0 then
--			else
--				env.info('PICKUPTASK:HasFailed() no cargo left. fail task.')
--				TaskHasFailed = true
--			end
--		end
--	end
	
--	env.info( 'PICKUPTASK:HasFailed() end : ' .. string.format ( "%s", TaskHasFailed and "true" or "false" ) )
	return TaskHasFailed
end

function PICKUPTASK:OnBoardCargo( ClientGroup, Cargos )
  env.info( "PickupTask:OnBoardCargo()" )
  
  local Valid = true
  
  Valid = _TransportValidateGroup( ClientGroup, "ClientGroup", Valid )
  
  if Valid then

		local CarrierPos = ClientGroup:getUnits()[1]:getPoint()
		local CarrierPosMove = ClientGroup:getUnits()[1]:getPoint()
		local CarrierPosOnBoard = ClientGroup:getUnits()[1]:getPoint()
		
		local CargoGroup = Group.getByName( Cargos[ self.CargoName ].CargoGroupName )
		trigger.action.setGroupAIOn( CargoGroup )

		local CargoUnits = CargoGroup:getUnits()
		local CargoPos = CargoUnits[1]:getPoint()

		
		local Points = {}
		
		env.info( 'CargoPos x = ' .. CargoPos.x .. " z = " .. CargoPos.z )
		env.info( 'CarrierPosMove x = ' .. CarrierPosMove.x .. " z = " .. CarrierPosMove.z )
		
		Points[#Points+1] = mist.ground.buildWP( CargoPos, "off road", 6 )

		env.info( 'Points[1] x = ' .. Points[1].x .. " y = " .. Points[1].y )
		
		if self.OnBoardSide == nil then
			self.OnBoardSide = CLIENT.ONBOARDSIDE.NONE
		end
		
		if    self.OnBoardSide == CLIENT.ONBOARDSIDE.LEFT then
			env.info( "TransportCargoOnBoard: Onboarding LEFT" )
			CarrierPosMove.z = CarrierPosMove.z - 50
			CarrierPosOnBoard.z = CarrierPosOnBoard.z - 5
			Points[#Points+1] = mist.ground.buildWP( CarrierPosMove, "diamond", 6 )
			Points[#Points+1] = mist.ground.buildWP( CarrierPosOnBoard, "diamond", 6 )
		elseif  self.OnBoardSide == CLIENT.ONBOARDSIDE.RIGHT then
			env.info( "TransportCargoOnBoard: Onboarding RIGHT" )
			CarrierPosMove.z = CarrierPosMove.z + 50
			CarrierPosOnBoard.z = CarrierPosOnBoard.z + 5
			Points[#Points+1] = mist.ground.buildWP( CarrierPosMove, "diamond", 6 )
			Points[#Points+1] = mist.ground.buildWP( CarrierPosOnBoard, "diamond", 6 )
		elseif  self.OnBoardSide == CLIENT.ONBOARDSIDE.BACK then
			env.info( "TransportCargoOnBoard: Onboarding BACK" )
			CarrierPosMove.x = CarrierPosMove.x - 50
			CarrierPosOnBoard.x = CarrierPosOnBoard.x - 5
			Points[#Points+1] = mist.ground.buildWP( CarrierPosMove, "diamond", 6 )
			Points[#Points+1] = mist.ground.buildWP( CarrierPosOnBoard, "diamond", 6 )
		elseif  self.OnBoardSide == CLIENT.ONBOARDSIDE.FRONT then
			env.info( "TransportCargoOnBoard: Onboarding FRONT" )
			CarrierPosMove.x = CarrierPosMove.x + 50
			CarrierPosOnBoard.x = CarrierPosOnBoard.x + 5
			Points[#Points+1] = mist.ground.buildWP( CarrierPosMove, "diamond", 6 )
			Points[#Points+1] = mist.ground.buildWP( CarrierPosOnBoard, "diamond", 6 )
		elseif  self.OnBoardSide == CLIENT.ONBOARDSIDE.NONE then
			env.info( "TransportCargoOnBoard: Onboarding CENTRAL" )
			Points[#Points+1] = mist.ground.buildWP( CarrierPos, "diamond", 6 )
		end
		env.info( "TransportCargoOnBoard: Routing " .. Cargos[ self.CargoName ].CargoGroupName )

		env.info( 'Points[2] x = ' .. Points[2].x .. " y = " .. Points[2].y )
		env.info( 'Points[3] x = ' .. Points[3].x .. " y = " .. Points[3].y )

		mist.goRoute( Cargos[ self.CargoName ].CargoGroupName, Points )     
  end  
  
  return Valid
  
end

--- Class
-- Modeling a sequence of STAGEs to fly back to the home base specified by an Arrival Zone.
GOHOMETASK = {
  ClassName = "GOHOMETASK",
}

--- Constructor
-- Creates a new GOHOMETASK.
-- @param self
-- @param LandingZoneName Name of the Zone where Home is located.
-- @author svenvandevelde
function GOHOMETASK:New( LandingZoneName )
  env.info( "GOHOMETASK:New()" .. self.ClassName )

  -- Child holds the inherited instance of the PICKUPTASK Class to the BASE class.
  local Child = BASE:Inherit( self, TASK:New() )

  local Valid = true
  
  Valid = _TransportValidateZone( LandingZoneName, "LandingZoneName", Valid )
    
  if  Valid then
    Child.Name = 'GoHomeTask'
    Child.TaskBriefing = "Task: Fly back to your home base located at " .. LandingZoneName .. ". Your co-pilot will provide you with the directions (required flight angle in degrees) and the distance (in km) to your home base."
    Child.LandingZoneName = LandingZoneName
    Child.Stages = { STAGEBRIEF:New(), STAGESTART:New(), STAGEROUTE:New(), STAGEARRIVE:New(), STAGEDONE:New() }
    Child:SetStage(1)
  end
  
  return Child
end

--- Class ROUTETASK
-- Modeling a sequence of STAGEs to fly back to the home base specified by an Arrival Zone.
ROUTETASK = {
  ClassName = "ROUTETASK",
}

--- Constructor
-- Creates a new ROUTETASK.
-- @param self
-- @param LandingZoneName Name of the Zone where the target is located.
-- @author svenvandevelde
function ROUTETASK:New( LandingZoneName )
  env.info( "ROUTETASK:New()" .. self.ClassName )

  -- Child holds the inherited instance of the PICKUPTASK Class to the BASE class.
  local Child = BASE:Inherit( self, TASK:New() )

  local Valid = true
  
  Valid = _TransportValidateZone( LandingZoneName, "LandingZoneName", Valid )
    
  if  Valid then
    Child.Name = 'RouteTask'
    Child.TaskBriefing = "Task: Fly to the zone located at " .. LandingZoneName .. ". Your co-pilot will provide you with the directions (required flight angle in degrees) and the distance (in km) to the target objective."
    Child.LandingZoneName = LandingZoneName
    Child.Stages = { STAGEBRIEF:New(), STAGESTART:New(), STAGEROUTE:New(), STAGEARRIVE:New(), STAGEDONE:New() }
    Child:SetStage(1)
  end
  
  return Child
end

--- Class NOTASK
-- Modeling a sequence of STAGEs to do nothing, but wait for the mission goal.
NOTASK = {
  ClassName = "NOTASK",
}

--- Constructor
-- Creates a new NOTASK.
-- @param self
-- @param LandingZoneName Name of the Zone where the target is located.
-- @author svenvandevelde
function NOTASK:New( )
  env.info( "ROUTETASK:New()" .. self.ClassName )

  -- Child holds the inherited instance of the PICKUPTASK Class to the BASE class.
  local Child = BASE:Inherit( self, TASK:New() )

  local Valid = true

  if  Valid then
    Child.Name = 'NoTask'
    Child.TaskBriefing = "Task: Execute your mission."
    Child.Stages = { STAGEBRIEF:New(), STAGESTART:New(), STAGEDONE:New() }
    Child:SetStage(1)
  end
  
  return Child
end

--- Class
-- Modeling a sequence of STAGEs to destroy groups.
DESTROYGROUPSTASK = {
  ClassName = "DESTROYGROUPSTASK",
}

--- Constructor
-- Creates a new DESTROYGROUPSTASK.
-- @param self
-- @param DestroyGroupNames Table of string containing the name of the groups to be destroyed before task is completed.
-- @author svenvandevelde
function DESTROYGROUPSTASK:New( DestroyGroupType, DestroyGroupNames )
  env.info( "DESTROYGROUPSTASK:New()" .. self.ClassName )

  -- Child holds the inherited instance of the PICKUPTASK Class to the BASE class.
  local Child = BASE:Inherit( self, TASK:New() )

  local Valid = true
   
  if  Valid then
    Child.Name = 'DestroyGroupsTask'
	Child.DestroyGroupCount = #DestroyGroupNames
	Child.DestroyGroupType = DestroyGroupType
	Child.DestroyGroupNames = DestroyGroupNames
    Child.TaskBriefing = "Task: Destroy " .. Child.DestroyGroupCount .. " " .. Child.DestroyGroupType .. "."
    Child.Stages = { STAGEBRIEF:New(), STAGESTART:New(), STAGEGROUPSDESTROYED:New(), STAGEDONE:New() }
    Child:SetStage(1)
  end
  
  return Child
end
